body,h1,h2,h3,h4,h5,h6,p,center,td,th,ul,dl,div {
  font-family: Arial, Geneva, Helvetica, sans-serif;
}

h1, div.title {
	text-align: center;
        font-size: 160%;
	font-weight: bold;
}
h2 {
	font-size: 120%;
}
h3 {
	font-size: 100%;
}
caption {
	font-weight: bold 
}
div.qindex {
	width: 100%;
	text-align: center;
	margin: 2px;
	padding: 2px;
	line-height: 140%;
}
div.navpath {
	width: 100%;
	background-color: #e8eef2;
	border: 1px solid #84b0c7;
	text-align: center;
	margin: 2px;
	padding: 2px;
	line-height: 140%;
}
div.navtab {
       background-color: #e8eef2;
       border: 1px solid #84b0c7;
       text-align: center;
       margin: 2px;
       margin-right: 15px;
       padding: 2px;
}
td.navtab {
       font-size: 70%;
}

div.qindex {
       display: none;
}
a.qindex {
       text-decoration: none;
       font-weight: bold;
       color: #1A419D;
}
a.qindex:visited {
       text-decoration: none;
       font-weight: bold;
       color: #1A419D
}
a.qindex:hover {
	text-decoration: none;
	background-color: #ddddff;
}
a.qindexHL {
	text-decoration: none;
	font-weight: bold;
	background-color: #6666cc;
	color: #ffffff;
	border: 1px double #9295C2;
}
a.qindexHL:hover {
	text-decoration: none;
	background-color: #6666cc;
	color: #ffffff;
}
a.qindexHL:visited {
	text-decoration: none; 
	background-color: #6666cc; 
	color: #ffffff 
}

a {
	text-decoration: none;
}

a.el {
	text-decoration: none; 
}
a.elRef {
	font-weight: bold 
}
a.code:link {
	text-decoration: none; 
	font-weight: normal; 
	color: #0000FF
}
a.code:visited {
	text-decoration: none; 
	font-weight: normal; 
	color: #0000FF
}
a.codeRef:link {
	font-weight: normal; 
	color: #0000FF
}
a.codeRef:visited {
	font-weight: normal; 
	color: #0000FF
}
a:hover {
	text-decoration: none; 	
	background-color: #f2f2ff 
}
dl.el {
	margin-left: -1cm 
}
.fragment {
       font-family: monospace, fixed;
       font-size: 95%;
}
pre.fragment {
	border: 1px solid #CCCCCC;
	background-color: #f5f5f5;
	margin-top: 4px;
	margin-bottom: 4px;
	margin-left: 2px;
	margin-right: 8px;
	padding-left: 6px;
	padding-right: 6px;
	padding-top: 4px;
	padding-bottom: 4px;
}
div.ah {
	font-weight: bold; 
	margin-bottom: 5px; 
	margin-top: 5px 
}

div.groupHeader {
       margin-left: 16px;
       margin-top: 12px;
       margin-bottom: 6px;
       font-weight: bold;
}
div.groupText {
	margin-left: 16px; 
	font-style: italic; 
	font-size: 90% 
}
body {
	background: white;
	color: black;
	margin-right: 20px;
	margin-left: 20px;
}
td.indexkey {
-moz-background-clip:border;
-moz-background-inline-policy:continuous;
-moz-background-origin:padding;
background:#F0F0F0 none repeat scroll 0 0;
color:black;
}
td.indexvalue {
-moz-background-clip:border;
-moz-background-inline-policy:continuous;
-moz-background-origin:padding;
background:#E4E4E4 none repeat scroll 0 0;
color:black;
}
tr.memlist {
	background-color: #f0f0f0; 
}
p.formulaDsp {
	text-align: center; 
}
img.formulaDsp {
}
img.formulaInl {
	vertical-align: middle; 
}
span.keyword       { color: #008000 }
span.keywordtype   { color: #604020 }
span.keywordflow   { color: #e08000 }
span.comment       { color: #800000 }
span.preprocessor  { color: #806020 }
span.stringliteral { color: #002080 }
span.charliteral   { color: #008080 }
span.vhdldigit     { color: #ff00ff }
span.vhdlchar      { color: #000000 }
span.vhdlkeyword   { color: #700070 }
span.vhdllogic     { color: #ff0000 }

.mdescLeft {
 border:1px none #E0E0E0;
 font-size:100%;
 margin:4px;
 padding:2px 0 0 8px;
 white-space:nowrap;
 width:180px;
}
.mdescRight {
 border:1px none #E0E0E0;
 font-size:100%;
 margin:4px;
 padding:2px 8px 0;
}
.memItemLeft {
        border:1px none #E0E0E0;
        font-size:100%;
        margin:4px;
        padding:2px 0 0 8px;
        white-space:nowrap;
        width:180px;
}
.memItemRight {
        border:1px none #E0E0E0;
        font-size:100%;
        margin:4px;
        padding:2px 8px 0;
}
.memTemplItemLeft {
	padding: 1px 0px 0px 8px;
	margin: 4px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-color: #E0E0E0;
	border-right-color: #E0E0E0;
	border-bottom-color: #E0E0E0;
	border-left-color: #E0E0E0;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
	background-color: #FAFAFA;
	font-size: 80%;
}
.memTemplItemRight {
	padding: 1px 8px 0px 8px;
	margin: 4px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-color: #E0E0E0;
	border-right-color: #E0E0E0;
	border-bottom-color: #E0E0E0;
	border-left-color: #E0E0E0;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
	background-color: #FAFAFA;
	font-size: 80%;
}
.memTemplParams {
	padding: 1px 0px 0px 8px;
	margin: 4px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-color: #E0E0E0;
	border-right-color: #E0E0E0;
	border-bottom-color: #E0E0E0;
	border-left-color: #E0E0E0;
	border-top-style: solid;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
	color: #606060;
	background-color: #FAFAFA;
	font-size: 80%;
}
.search { 
	color: #003399;
	font-weight: bold;
}
form.search {
	margin-bottom: 0px;
	margin-top: 0px;
}
input.search {
	font-size: 75%;
	color: #000080;
	font-weight: normal;
	background-color: #e8eef2;
}
td.tiny {
	font-size: 75%;
}
a {
	color: #1A41A8;
}
a:visited {
	color: #2A3798;
}
.dirtab {
	padding: 4px;
	border-collapse: collapse;
	border: 1px solid #84b0c7;
}
th.dirtab {
	background: #e8eef2;
	font-weight: bold;
}
hr {
border: 0;
color: #a0a0a0;
background-color: #ccc;
height: 1px;
width: 100%;
text-align: left;
margin: 34px 0px 34px 0px;

}

/* Style for detailed member documentation */
.memtemplate {
	font-size: 80%;
	color: #606060;
	font-weight: normal;
	margin-left: 3px;
} 
.memnav { 
	background-color: #e8eef2;
	border: 1px solid #84b0c7;
	text-align: center;
	margin: 2px;
	margin-right: 15px;
	padding: 2px;
}
.memitem {
	padding: 4px;
}
.memname {
	white-space: nowrap;
	font-weight: bold;
	font-size: 105%;



}
.memdoc{
	padding-left: 10px;
}
.memproto {
 background-color:#EEEEEE;
 border:1px solid #DDDDDD;
 font-weight:bold;
 padding:3px 0 3px 10px;
}
.paramkey {
	text-align: right;
}
.paramtype {
	white-space: nowrap;
}
.paramname {
	font-style: italic;
	white-space: nowrap;
}
/* End Styling for detailed member documentation */

/* for the tree view */
.ftvtree {
	font-family: sans-serif;
	margin:0.5em;
}
.directory { 
	font-size: 9pt; 
	font-weight: bold; 
}
.directory h3 { 
	margin: 0px; 
	margin-top: 1em; 
	font-size: 11pt; 
}
.directory > h3 { 
	margin-top: 0; 
}
.directory p { 
	margin: 0px; 
	white-space: nowrap; 
}
.directory div { 
	display: none; 
	margin: 0px; 
}
.directory img { 
	vertical-align: -30%; 
}

#top
{
   visibility: hidden;
   display: none;
}

table.mainIndex {
   border-spacing: 0px;
   width: 100%;
   padding: 0px;
   background-color: #FAFAFA;
   border-width: 1px;
   border-style: solid;
   border-color: #E0E0E0;
}

table.mainIndex th {
   width: 33%;
   border-width: 1px 0px 1px 0px;
   padding: 2px;
   border-style: solid;
   border-color: #666;
   color:white;
   background-color:#555;
   background-image:url('gradient.png')};
   background-repeat: repeat-x;
   font-size: 100%;
}

table.mainIndex td {
   border-style: none;
   vertical-align: top;
   padding-top: 1em;
}

table.qt-style th {
    background: #96E066;
    color: black;
}
table.qt-style {
    padding-top: 0px;
    padding-bottom: 0px;
    padding-left: 0px;
    padding-right: 0px;
    border: none;
    background: none;
}

table.qt-style tr:nth-child(odd) {
    background: #e4e4e4;
    color: black;
}

table.qt-style tr:nth-child(even) {
    background: #f0f0f0;
    color: black;
}

/* Styles for settings language page */
div.settingsLanguageElement {
   border-style: solid;
   border-width: 1px;
   border-color: #E0E0E0;
   margin-bottom: 4em;
   padding: 8px;
   background-color: #FDFDFD;
}

div.settingsLanguageElement h2 {
   background-color: #ffd429;
   margin-top: 0px;
   padding: 8px;
   -moz-border-radius: 8px 8px 8px 8px;
}

table.settingsLanguageAttributes {
   border-spacing: 1px;
   width: 100%;
   padding: 4px;
   background-color: #FAFAFA;
   border-width: 1px;
   border-style: solid;
   border-color: #E0E0E0;
   -moz-border-radius: 8px 8px 8px 8px;
}

table.settingsLanguageAttributes th {
   background-color: #dddddd;
   padding: 4px;
   border-style: none;
   -moz-border-radius: 8px 8px 8px 8px;
}

table.settingsLanguageAttributes td {
   border-style: none;
   vertical-align: top;
   padding-top: 1em;
}
/* End of styles for settings language page */

td.generator {
   font-size: 75%;
   color: #a0a0a0;
}
