/* This file is part of Maliit framework
 *
 * Copyright (C) 2012 Canonical Ltd
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * and appearing in the file LICENSE.LGPL included in the packaging
 * of this file.
 */

#ifndef UT_MIMONSCREENPLUGINS_H
#define UT_MIMONSCREENPLUGINS_H

#include <QtTest/QtTest>
#include <QObject>

class Ut_MImOnScreenPlugins
    : public QObject
{
    Q_OBJECT

private Q_SLOTS:
    void initTestCase();
    void cleanupTestCase();

    void init();
    void cleanup();

    void testActiveAndEnabledSubviews_data();
    void testActiveAndEnabledSubviews();
};

#endif
