/* * This file is part of Maliit framework *
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * and appearing in the file LICENSE.LGPL included in the packaging
 * of this file.
 */

#ifndef NAMESPACEINTERNAL_H
#define NAMESPACEINTERNAL_H

//! \internal
namespace Maliit { namespace Internal {

    //! Name of the input method hints stored in our update map.
    const char* const inputMethodHints = "maliit-inputmethod-hints";

}} // namespace Internal, Maliit

#endif // NAMESPACEINTERNAL_H
