/* * This file is part of Maliit framework *
 *
 * Copyright (C) 2022 Pekka Vuorela
 * All rights reserved.
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * and appearing in the file LICENSE.LGPL included in the packaging
 * of this file.
 */

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(lcMaliitFw)
